#include <Pokitto.h>
#include <Tilemap.hpp>
#include "sprites.h"
#include "maps.h"
#include "palette.h"

#define PLAYER_TOP_DISTANCE     24
#define PLAYER_HEIGHT           16
#define SCREEN_HEIGHT           176
#define JUMP_HEIGHT             300.0
#define OFFSET_START            (SCREEN_HEIGHT - PLAYER_TOP_DISTANCE - PLAYER_HEIGHT)

int playerX = 0;
int playerY = 0;
int playerDrawY = 0;
int playerXd = 1;
float r = 0;
int stageHeight = 0;
int offsetY = 0;
int cameraOffset = 0;

int main(){
    using PC=Pokitto::Core;
    using PD=Pokitto::Display;
    using PB=Pokitto::Buttons;
    
    PC::begin();
    PD::loadRGBPalette(auroraPal);
    PD::setFont(fontTiny);
    
    Tilemap tilemap;
    tilemap.set(stage[0], stage[1], stage + 2);
    for(int i = 0; i < sizeof(tiles) / (POK_TILE_W * POK_TILE_H); i++) {
        tilemap.setTile(i, POK_TILE_W, POK_TILE_H, tiles + i * POK_TILE_W * POK_TILE_H);
    }
    
    stageHeight = POK_TILE_H * stage[1];

    Sprite sprite;
    sprite.play(player, Player::jumping);
    
    while(PC::isRunning()) {
        if(PC::update()) {
            if (!PB::rightBtn()) {
                // Update player coordinates
                if (playerX >= 160) {
                    playerXd = -2;
                } else if (playerX == 0) {
                    playerXd = 2;
                }
                
                playerX += playerXd;
                playerY = (int) (JUMP_HEIGHT * abs(sin(r)));
                r += 0.015 * PI;
                if (r >= PI) {
                    r = 0;
                }
                
                // Calculate offsets and camera position
                if (playerY < OFFSET_START) {
                    offsetY = 0;
                    playerDrawY = SCREEN_HEIGHT - playerY - PLAYER_HEIGHT;
                } else {
                    offsetY = playerY - OFFSET_START;
                    playerDrawY = PLAYER_TOP_DISTANCE;
                }
                cameraOffset = -stageHeight + SCREEN_HEIGHT + offsetY;
            }
            
            // Draw
            tilemap.draw(0, cameraOffset);
            sprite.draw(playerX, playerDrawY, false, false, 0);
            
            // Debug
            PD::setColor(64);
            PD::bgcolor = 47;
            PD::print(180, 0, PC::fps_counter);
        }
    }
    
    return 0;
}
